@@ -22,11 +22,15 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+import logging
 
-from fer.fer import FER
-from fer.classes import Video
+from .classes import Video
+from .fer import FER
 
-__version__ = "21.0.2"
+log = logging.getLogger("fer")
+log.setLevel(logging.INFO)
+
+__version__ = "22.1.1"
 
 __title__ = "fer"
 __description__ = "Facial expression recognition from images"
