@@ -1,58 +1,31 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-import logging
-
 import csv
+import logging
 import os
 import re
-import requests
-import time
-from typing import Union, Optional
-from zipfile import ZipFile
 from pathlib import Path
+from typing import Optional, Union
+from zipfile import ZipFile
+
 import cv2
-import numpy as np
 import pandas as pd
 
-logging.getLogger(__name__)
+from tqdm import tqdm
+from tqdm.contrib.logging import logging_redirect_tqdm
 
+from .utils import draw_annotations
 
-class Peltarion_Emotion_Classifier(object):
-    def __init__(self, url, token, shape=(48, 48)):
-        self.url = url
-        self.token = token
-        self.shape = shape
-
-    @staticmethod
-    def _unnormalize_face(
-        gray_face: np.ndarray, shape: tuple = (48, 48), v2: bool = True
-    ) -> object:
-        gray_face = gray_face.reshape(shape)
-        if v2:
-            gray_face = gray_face / 2.0
-            gray_face = gray_face + 0.5
-        gray_face = gray_face * 255.0
-        return gray_face
-
-    def predict(self, gray_face) -> list:
-        """Gray face to emotions with Peltarion REST API"""
-        instance_path = os.environ.get("FLASK_INSTANCE_PATH", os.getcwd())
-        temp_filepath = os.path.join(instance_path, "tmp_01.npy")
-        gray_face = gray_face.reshape(48, 48, 1).astype(np.float32)
-        np.save(temp_filepath, gray_face)
-        headers = {"Authorization": "Bearer " + self.token}
-        files = {"image": open(temp_filepath, "rb")}
-        response = requests.post(self.url, headers=headers, files=files).json()
-        try:
-            emotion = response["emotion"]
-        except:
-            logging.error(f"{response} is not a valid response")
-        return emotion
+log = logging.getLogger("fer")
 
 
 class Video(object):
     def __init__(
-        self, video_file:str, outdir:str="output", first_face_only:bool=True, tempfile:Optional[str]=None
+        self,
+        video_file: str,
+        outdir: str = "output",
+        first_face_only: bool = True,
+        tempfile: Optional[str] = None,
     ):
         """Video class for extracting and saving frames for emotion detection.
         :param video_file - str
@@ -70,7 +43,7 @@ class Video(object):
         self.outdir = outdir
 
         if not first_face_only:
-            logging.error("Only single-face charting is implemented")
+            log.error("Only single-face charting is implemented")
         self.first_face_only = first_face_only
         self.tempfile = tempfile
         self.filepath = video_file
@@ -149,6 +122,7 @@ class Video(object):
 
     def to_csv(self, data, filename="data.csv"):
         """Save data to csv"""
+
         def key(item):
             key_pat = re.compile(r"^(\D+)(\d+)$")
             m = key_pat.match(item)
@@ -164,9 +138,66 @@ class Video(object):
             writer.writerows(dictlist)
         return dictlist
 
+    def _close_video(self, outfile, save_frames, zip_images):
+        self.cap.release()
+        if self.display or self.save_video:
+            self.videowriter.release()
+
+        if self.save_video:
+            log.info("Completed analysis: saved to {}".format(self.tempfile or outfile))
+            if self.tempfile:
+                os.replace(self.tempfile, outfile)
+
+        if save_frames and zip_images:
+            log.info("Starting to Zip")
+            outdir = Path(self.outdir)
+            zip_dir = outdir / "images.zip"
+            images = sorted(list(outdir.glob("*.jpg")))
+            total = len(images)
+            i = 0
+            with ZipFile(zip_dir, "w") as zip:
+                for file in images:
+                    zip.write(file, arcname=file.name)
+                    os.remove(file)
+                    i += 1
+                    if i % 50 == 0:
+                        log.info(f"Compressing: {i*100 // total}%")
+            log.info("Zip has finished")
+
+    def _offset_detection_box(self, faces, detection_box):
+        for face in faces:
+            original_box = face.get("box")
+            face["box"] = (
+                original_box[0] + detection_box.get("x_min"),
+                original_box[1] + detection_box.get("y_min"),
+                original_box[2],
+                original_box[3],
+            )
+        return faces
+
+    def _increment_frames(self, frame, faces, video_id, root):
+        # Save images to `self.outdir`
+        imgpath = os.path.join(
+            self.outdir, (video_id or root) + str(self.frameCount) + ".jpg"
+        )
+
+        if self.annotate_frames:
+            frame = draw_annotations(frame, faces, boxes=True, scores=True)
+
+        if self.save_frames:
+            cv2.imwrite(imgpath, frame)
+
+        if self.display:
+            cv2.imshow("Video", frame)
+
+        if self.save_video:
+            self.videowriter.write(frame)
+
+        self.frameCount += 1
+
     def analyze(
         self,
-        detector, # fer.FER instance
+        detector,  # fer.FER instance
         display: bool = False,
         output: str = "csv",
         frequency: Optional[int] = None,
@@ -177,12 +208,12 @@ class Video(object):
         save_video: bool = True,
         annotate_frames: bool = True,
         zip_images: bool = True,
-        detection_box: Optional[dict] = None
+        detection_box: Optional[dict] = None,
     ) -> list:
         """Recognize facial expressions in video using `detector`.
-        
+
         Args:
-        
+
             detector (fer.FER): facial expression recognizer
             display (bool): show images with cv2.imshow
             output (str): csv or pandas
@@ -191,22 +222,27 @@ class Video(object):
             save_fps (bool): inference frequency = video fps // save_fps
             video_id (str): filename for saving
             save_frames (bool): saves frames to directory
-            save_videos (bool): saves output video
+            save_video (bool): saves output video
             annotate_frames (bool): add emotion labels
             zip_images (bool): compress output
-            detection_box (dict): dict with bounding box for subimage (xmin, xmax, ymin, ymax)        
+            detection_box (dict): dict with bounding box for subimage (xmin, xmax, ymin, ymax)
 
         Returns:
-        
+
             data (list): list of results
-            
+
         """
-        data = []
+        frames_emotions = []
         if frequency is None:
             frequency = 1
         else:
             frequency = int(frequency)
 
+        self.display = display
+        self.save_frames = save_frames
+        self.save_video = save_video
+        self.annotate_frames = annotate_frames
+
         results_nr = 0
 
         # Open video
@@ -216,7 +252,7 @@ class Video(object):
         pos_frames = self.cap.get(cv2.CAP_PROP_POS_FRAMES)
         assert int(pos_frames) == 0, "Video not at index 0"
 
-        frameCount = 0
+        self.frameCount = 0
         height, width = (
             int(self.cap.get(cv2.CAP_PROP_FRAME_HEIGHT)),
             int(self.cap.get(cv2.CAP_PROP_FRAME_WIDTH)),
@@ -228,158 +264,78 @@ class Video(object):
 
         if save_fps is not None:
             frequency = fps // save_fps
-            logging.info("Saving every {} frames".format(frequency))
+            log.info("Saving every {} frames".format(frequency))
 
-        logging.info(
+        log.info(
             "{:.2f} fps, {} frames, {:.2f} seconds".format(fps, length, length / fps)
         )
 
-        capture_duration = 1000 / fps
-
-        if save_frames:
+        if self.save_frames:
             os.makedirs(self.outdir, exist_ok=True)
-            logging.info(f"Making directories at {self.outdir}")
+            log.info(f"Making directories at {self.outdir}")
         root, ext = os.path.splitext(os.path.basename(self.filepath))
         outfile = os.path.join(self.outdir, f"{root}_output{ext}")
 
         if save_video:
-            videowriter = self._save_video(outfile, fps, width, height)
+            self.videowriter = self._save_video(outfile, fps, width, height)
+
+        with logging_redirect_tqdm():
+            pbar = tqdm(total=length, unit="frames")
 
         while self.cap.isOpened():
-            start_time = time.time()
             ret, frame = self.cap.read()
             if not ret:  # end of video
                 break
-            if frameCount % frequency != 0:
-                frameCount += 1
+
+            if frame is None:
+                log.warn("Empty frame")
+                continue
+
+            if self.frameCount % frequency != 0:
+                self.frameCount += 1
                 continue
 
             if detection_box is not None:
-                try:
-                    frame = self._crop(frame, detection_box)
-                except Exception as e:
-                    logging.error(e)
-                    break
+                frame = self._crop(frame, detection_box)
 
-            padded_frame = detector.pad(frame)
+            # Get faces and detect emotions; coordinates are for unpadded frame
             try:
-                # Get faces with emotions
-                faces = detector.detect_emotions(padded_frame)
-                if detection_box is not None:
-                    try:
-                        for face in faces:
-                            original_box = face.get("box")
-                            face["box"] = (original_box[0] + detection_box.get("x_min"), original_box[1] + detection_box.get("y_min"), original_box[2], original_box[3])
-                    except Exception as e:
-                        logging.error(e)
-
+                faces = detector.detect_emotions(frame)
             except Exception as e:
-                logging.error(e)
+                log.error(e)
                 break
 
-            # Save images to `self.outdir`
-            imgpath = os.path.join(
-                self.outdir, (video_id or root) + str(frameCount) + ".jpg"
-            )
-            if save_frames and not annotate_frames:
-                cv2.imwrite(imgpath, frame)
+            # Offset detection_box to include padding
+            if detection_box is not None:
+                faces = self._offset_detection_box(faces, detection_box)
 
-            if display or save_video or annotate_frames:
-                assert isinstance(faces, list), type(faces)
-                for face in faces:
-                    bounding_box = face["box"]
-                    emotions = face["emotions"]
+            self._increment_frames(frame, faces, video_id, root)
 
-                    cv2.rectangle(
-                        frame,
-                        (bounding_box[0] - 40, bounding_box[1] - 40),
-                        (
-                            bounding_box[0] - 40 + bounding_box[2],
-                            bounding_box[1] - 40 + bounding_box[3],
-                        ),
-                        (0, 155, 255),
-                        2,
-                    )
+            if cv2.waitKey(1) & 0xFF == ord("q"):
+                break
 
-                    for idx, (emotion, score) in enumerate(emotions.items()):
-                        color = (211, 211, 211) if score < 0.01 else (0, 255, 0)
-                        emotion_score = "{}: {}".format(
-                            emotion, "{:.2f}".format(score) if score > 0.01 else ""
-                        )
-                        cv2.putText(
-                            frame,
-                            emotion_score,
-                            (
-                                bounding_box[0] - 40,
-                                bounding_box[1] - 40 + bounding_box[3] + 30 + idx * 15,
-                            ),
-                            cv2.FONT_HERSHEY_SIMPLEX,
-                            0.5,
-                            color,
-                            1,
-                            cv2.LINE_AA,
-                        )
-                    if display:
-                        cv2.imshow("Video", frame)
-                    if save_frames and annotate_frames:
-                        cv2.imwrite(imgpath, frame)
-                    if save_video:
-                        videowriter.write(frame)
-                    results_nr += 1
-
-                if display or save_video:
-                    remaining_duration = max(
-                        1, int((time.time() - start_time) * 1000 - capture_duration)
-                    )
-                    if cv2.waitKey(remaining_duration) & 0xFF == ord("q"):
-                        break
-                else:
-                    if cv2.waitKey(1) & 0xFF == ord("q"):
-                        break            
-
-            frameCount += 1
             if faces:
-                data.append(faces)
+                frames_emotions.append(faces)
+
+            results_nr += 1
             if max_results and results_nr > max_results:
                 break
 
-        self.cap.release()
-        if display or save_video:
-            videowriter.release()
-            if save_video:
-                logging.info(
-                    "Completed analysis: saved to {}".format(self.tempfile or outfile)
-                )
-                if self.tempfile:
-                    os.replace(self.tempfile, outfile)
+            pbar.update(1)
 
-        if save_frames and zip_images:
-            print("Starting to Zip")
-            outdir = Path(self.outdir)
-            zip_dir = outdir / 'images.zip'
-            images = sorted(list(outdir.glob("*.jpg")))
-            total = len(images)
-            i = 0
-            with ZipFile(zip_dir, 'w') as zip:
-                for file in images:
-                    zip.write(file, arcname=file.name)
-                    os.remove(file)
-                    i += 1
-                    if i%50 == 0: print(f"Compressing: {i*100 // total}%")
-            print("Zip has finished")
+        pbar.close()
+        self._close_video(outfile, save_frames, zip_images)
 
-        if output == "csv":
-            return self.to_csv(data)
-        elif output == "pandas":
-            return self.to_pandas(data)
-        else:
-            raise NotImplementedError(f"{output} is not supported")
-        return data
+        return self.to_format(frames_emotions, output)
 
-    def _save_video(self, outfile:str, fps: int, width: int, height: int):
+    def to_format(self, data, format):
+        methods_lookup = {"csv": self.to_csv, "pandas": self.to_pandas}
+        return methods_lookup[format](data)
+
+    def _save_video(self, outfile: str, fps: int, width: int, height: int):
         if os.path.isfile(outfile):
             os.remove(outfile)
-            logging.info("Deleted pre-existing {}".format(outfile))
+            log.info("Deleted pre-existing {}".format(outfile))
         if self.tempfile and os.path.isfile(self.tempfile):
             os.remove(self.tempfile)
         fourcc = cv2.VideoWriter_fourcc("m", "p", "4", "v")
@@ -389,14 +345,12 @@ class Video(object):
         return videowriter
 
     @staticmethod
-    def _crop(self, frame, detection_box):
+    def _crop(frame, detection_box):
         crop_frame = frame[
-                     detection_box.get("y_min"): detection_box.get("y_max"),
-                     detection_box.get("x_min"): detection_box.get("x_max")]
+            detection_box.get("y_min") : detection_box.get("y_max"),
+            detection_box.get("x_min") : detection_box.get("x_max"),
+        ]
         return crop_frame
 
     def __del__(self):
-        try:
-            cv2.destroyAllWindows()
-        except:
-            pass
+        cv2.destroyAllWindows()
