@@ -23,36 +23,31 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import logging
-
 # IMPORTANT:
 #
 # This code is derived from Iván de Paz Centeno's implementation of MTCNN
 # (https://github.com/ipazc/mtcnn/) and Octavia Arriaga's facial expression recognition repo
 # (https://github.com/oarriaga/face_classification).
 #
-import os
+import logging
+import pkg_resources
 import sys
+from typing import Sequence, Tuple, Union
 
 import cv2
 import numpy as np
-import pkg_resources
-import tensorflow.compat.v1 as tf
-tf.disable_v2_behavior()
+
 from tensorflow.keras.models import load_model
 
-from fer.exceptions import InvalidImage
+from .exceptions import InvalidImage
+
+log = logging.getLogger("fer")
 
-from typing import Sequence, Tuple, Union
 NumpyRects = Union[np.ndarray, Sequence[Tuple[int, int, int, int]]]
 
 __author__ = "Justin Shenk"
 
-logging.basicConfig(
-    format="%(asctime)s,%(msecs)d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s",
-    datefmt="%d-%m-%Y:%H:%M:%S",
-    level=logging.INFO,
-)
+PADDING = 40
 
 
 class FER(object):
@@ -109,33 +104,25 @@ class FER(object):
         emotion_model = pkg_resources.resource_filename(
             "fer", "data/emotion_model.hdf5"
         )
-        self.config = tf.compat.v1.ConfigProto(log_device_placement=False)
-        self.config.gpu_options.allow_growth = True
-
-        self.__graph = tf.Graph()
-
-        self.__session = tf.compat.v1.Session(config=self.config, graph=self.__graph)
-
-        # with tf.Session(graph=K.get_session().graph, config=self.config) as sess:
         self.__emotion_classifier = load_model(emotion_model, compile=compile)
-        self.__emotion_classifier._make_predict_function()
+        self.__emotion_classifier.make_predict_function()
         self.__emotion_target_size = self.__emotion_classifier.input_shape[1:3]
 
-        logging.debug("Emotion model: {}".format(emotion_model))
+        log.debug("Emotion model: {}".format(emotion_model))
 
     @staticmethod
     def pad(image):
+        """Pad image."""
         row, col = image.shape[:2]
         bottom = image[row - 2 : row, 0:col]
         mean = cv2.mean(bottom)[0]
 
-        bordersize = 40
         padded_image = cv2.copyMakeBorder(
             image,
-            top=bordersize,
-            bottom=bordersize,
-            left=bordersize,
-            right=bordersize,
+            top=PADDING,
+            bottom=PADDING,
+            left=PADDING,
+            right=PADDING,
             borderType=cv2.BORDER_CONSTANT,
             value=[mean, mean, mean],
         )
@@ -144,7 +131,7 @@ class FER(object):
     @staticmethod
     def depad(image):
         row, col = image.shape[:2]
-        return image[40 : row - 40, 40 : col - 40]
+        return image[PADDING : row - PADDING, PADDING : col - PADDING]
 
     @staticmethod
     def tosquare(bbox):
@@ -159,7 +146,7 @@ class FER(object):
             y -= diff // 2
             h += diff
         if w != h:
-            logging.debug(f"{w} is not {h}")
+            log.debug(f"{w} is not {h}")
 
         return (x, y, w, h)
 
@@ -194,9 +181,16 @@ class FER(object):
         return x
 
     def __apply_offsets(self, face_coordinates):
+        """Offset face coordinates with padding before classification.
+        x1, x2, y1, y2 = 0, 100, 0, 100 becomes -10, 110, -10, 110
+        """
         x, y, width, height = face_coordinates
         x_off, y_off = self.__offsets
-        return (x - x_off, x + width + x_off, y - y_off, y + height + y_off)
+        x1 = x - x_off
+        x2 = x + width + x_off
+        y1 = y - y_off
+        y2 = y + height + y_off
+        return x1, x2, y1, y2
 
     @staticmethod
     def _get_labels():
@@ -210,7 +204,9 @@ class FER(object):
             6: "neutral",
         }
 
-    def detect_emotions(self, img: np.ndarray, face_rectangles: NumpyRects = None) -> list:
+    def detect_emotions(
+        self, img: np.ndarray, face_rectangles: NumpyRects = None
+    ) -> list:
         """
         Detects bounding boxes from the specified image with ranking of emotions.
         :param img: image to process (BGR or gray)
@@ -221,53 +217,64 @@ class FER(object):
 
         emotion_labels = self._get_labels()
 
-        if (face_rectangles is None):
+        if face_rectangles is None:
             face_rectangles = self.find_faces(img, bgr=True)
 
         gray_img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
+        gray_img = self.pad(gray_img)
 
         emotions = []
+        gray_faces = []
+
         for face_coordinates in face_rectangles:
             face_coordinates = self.tosquare(face_coordinates)
+
+            # offset to expand bounding box
+            # Note: x1 and y1 can be negative
             x1, x2, y1, y2 = self.__apply_offsets(face_coordinates)
 
-            if y1 < 0 or x1 < 0:
-                gray_img = self.pad(gray_img)
-                x1 += 40
-                x2 += 40
-                y1 += 40
-                y2 += 40
-                x1 = np.clip(x1, a_min=0, a_max=None)
-                y1 = np.clip(y1, a_min=0, a_max=None)
+            # account for padding in bounding box coordinates
+            x1 += PADDING
+            y1 += PADDING
+            x2 += PADDING
+            y2 += PADDING
+            x1 = np.clip(x1, a_min=0, a_max=None)
+            y1 = np.clip(y1, a_min=0, a_max=None)
 
-            gray_face = gray_img[y1:y2, x1:x2]
+            gray_face = gray_img[max(0, y1) : y2, max(0, x1) : x2]
 
             try:
                 gray_face = cv2.resize(gray_face, self.__emotion_target_size)
             except Exception as e:
-                print("{} resize failed: {}".format(gray_face.shape, e))
+                log.warn("{} resize failed: {}".format(gray_face.shape, e))
                 continue
-            
+
             # Local Keras model
             gray_face = self.__preprocess_input(gray_face, True)
-            gray_face = np.expand_dims(gray_face, 0)
-            gray_face = np.expand_dims(gray_face, -1)
+            gray_faces.append(gray_face)
 
-            emotion_prediction = self.__emotion_classifier.predict(gray_face)[0]
+        # predict all faces
+        if not len(gray_faces):
+            return emotions
+
+        emotion_predictions = self.__emotion_classifier(np.array(gray_faces))
+        for face_idx, face in enumerate(emotion_predictions):
             labelled_emotions = {
                 emotion_labels[idx]: round(float(score), 2)
-                for idx, score in enumerate(emotion_prediction)
+                for idx, score in enumerate(face)
             }
 
             emotions.append(
-                dict(box=face_coordinates, emotions=labelled_emotions)
+                dict(box=face_rectangles[face_idx], emotions=labelled_emotions)
             )
 
         self.emotions = emotions
 
         return emotions
 
-    def top_emotion(self, img: np.ndarray)-> Tuple[Union[str,None], Union[float,None]]:
+    def top_emotion(
+        self, img: np.ndarray
+    ) -> Tuple[Union[str, None], Union[float, None]]:
         """Convenience wrapper for `detect_emotions` returning only top emotion for first face in frame.
         :param img: image to process
         :return: top emotion and score (for first face in frame) or (None, None)
